import javax.management.openmbean.CompositeData;
import javax.management.openmbean.OpenDataException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class JMXDominoStatistic implements JMXDominoStatisticMBean {
    private CompositeData platformMemoryRam = null;
    private float databaseBufferPoolPerCentReadsInBuffer = 0;

    public JMXDominoStatistic() {

    }

    @Override
    public CompositeData getPlatformMemoryRam() {
        return platformMemoryRam;
    }

    @Override
    public float getDatabaseBufferPoolPerCentReadsInBuffer() {
        return databaseBufferPoolPerCentReadsInBuffer;
    }

    private void setObjectValue(Object obj, Class clazz, String method, String value) {
//        System.out.println("method = " + method);
//        System.out.println("value = " + value);

        Optional<Method> methodToFind =
                Arrays.stream(clazz.getDeclaredMethods()).
                        filter(m -> method.equals(m.getName())).
                        findFirst();

        if (methodToFind.isPresent()) {
            // invoke method or any logic needed
            try {
                methodToFind.get().invoke(obj, getFloatFromString(value));
            } catch (IllegalAccessException e) {
                //TODO:
                e.printStackTrace();
            } catch (InvocationTargetException e) {
                //TODO:
                e.printStackTrace();
            }
        }
    }

    public void update(Map<String, String> statValues) {
        PlatformMemoryRAM platformMemoryRAM = new PlatformMemoryRAM();

        statValues.entrySet().stream()
                .filter(x -> x.getKey().startsWith("Platform.Memory.RAM."))
                .forEach(entry -> setObjectValue(
                        platformMemoryRAM,
                        PlatformMemoryRAM.class,
                        "set" + entry.getKey().replace("Platform.Memory.RAM.", "").replaceAll("\\.", ""),
                        entry.getValue()
                        )
                );

        if(statValues.containsKey("Database.Database.BufferPool.PerCentReadsInBuffer")){
            databaseBufferPoolPerCentReadsInBuffer = getFloatFromString(statValues.get("Database.Database.BufferPool.PerCentReadsInBuffer"));
        }

        try {
//            System.out.println("platformMemoryRAM = " + platformMemoryRAM);

            platformMemoryRam = platformMemoryRAM.getCompositeData();
        } catch (OpenDataException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    private float getFloatFromString(String prop) {
        String decimalSeparator = ",";
        String groupSeparatorLinux = ".";
        String groupSeparatorWindows = " ";
        float ret = Float.parseFloat(prop.replace(groupSeparatorLinux, "").replace(groupSeparatorWindows, "").replace(decimalSeparator, "."));

//        System.out.println("prop = " + prop);
//        System.out.println("ret = " + ret);

        return ret;
    }
}
